@@ -4,6 +4,7 @@
 #include <ut/ut.hpp>
 
 #include "glaze/base64/base64.hpp"
+#include "glaze/util/progress_bar.hpp"
 
 using namespace ut;
 
@@ -37,4 +38,16 @@ suite base64_roundtrip_tests = [] {
    };
 };
 
+suite progress_bar_tests = [] {
+   "progress bar 30%"_test = [] {
+      glz::progress_bar bar{.width = 12, .completed = 3, .total = 10, .time_taken = 30.0};
+      expect(bar.string() == "[===-------] 30% | ETA: 1m 10s | 3/10") << bar.string();
+   };
+
+   "progress bar 100%"_test = [] {
+      glz::progress_bar bar{.width = 12, .completed = 10, .total = 10, .time_taken = 30.0};
+      expect(bar.string() == "[==========] 100% | ETA: 0m 0s | 10/10") << bar.string();
+   };
+};
+
 int main() { return 0; }
